             (* +------------------------------------+
                |  Sous-somme minimale d'un tableau  |
                +------------------------------------+ *)

(* tableau alatoire *)
let random_vect n a b =
  let t = make_vect n 0 in
  for i=0 to n-1 do t.(i) <- a + random__int(b-a+1) done;
  t
;;

(* algorithme en n^3 *)

let coupe_3 t i j =
  let x = ref(t.(i)) in
  for k = i+1 to j do x := !x + t.(k) done;
  !x
;;

let minisomme_3 t =
  let n = vect_length(t) in
  let s = ref t.(0)      in
  for i=0 to n-1 do for j=i to n-1 do s := min !s (coupe_3 t i j) done done;
  !s
;;


(* algorithme en n^2 *)

let coupe_2 t i =
  let x = ref(t.(i)) and s = ref(t.(i))  in
  for j=i+1 to vect_length(t)-1 do
    x := !x + t.(j);
    s := min !s !x
  done;
  !s
;;

let minisomme_2 t =
  let s = ref(coupe_2 t 0) in
  for i=1 to vect_length(t)-1 do s := min !s (coupe_2 t i) done;
  !s
;; 


(* algorithme en n *)

let minisomme_1 t =
  let x = ref(t.(0)) and s = ref(t.(0)) in
  for i=1 to vect_length(t)-1 do
    x := min (!x + t.(i)) t.(i);
    s := min !s !x
  done;
  !s
;;


(* meilleure approximation *)

let approx t k =
  let r = ref(t.(0)) in
  for i=0 to vect_length(t)-1 do
    let s = ref(0) in
    for j=i to vect_length(t)-1 do
      s := !s + t.(j);
      if abs(!s - k) < abs(!r - k) then r := !s;
    done
  done;
  !r
;;
